/*********************************************************************
*                                                                    *
*                      Software License Agreement                    *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products.                                                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

#include "sound.h"

static char info1[] = "Gain:. Echo:..";
static char info2[] = "Samples Per Int:.";
static int last_back_util = 0;
static int last_fore_util = 0;

void reset_utilization() {
  last_back_util = -1;
  last_fore_util = -1;
}

/*
 * Update line 1 of the control display
 */
void update_control_display1() {
  static char display1[] = { char_pos, 0, 1 };
  static char clear_line[] = {clear_to_eol};
  info1[5]  = '0' + gain;
  info1[12] = '0' + (delay > 9);
  info1[13] = '0' + ((delay > 9) ? delay - 10 : delay);
  lcd_dump(display1,3);
  write(0, info1, 14);
  // lcd_dump(clear_line, 1);
}

/*
 * Update line 2 of the control display
 * (shared with foreground utilization)
 */
void update_control_display2() {
  static char display2[] = { char_pos, 0, 2 };
  static char clear_line[] = {clear_to_eol};
  info2[16] = '0' + sample_set;
  if ((status.record == 0 ) && (status.play == 0)) {
    lcd_dump(display2,3);
    write(0, info2, 17);
    lcd_dump(clear_line, 1);
  } else {
    lcd_dump(display2,3);
    lcd_dump(clear_line, 1);
  }
   
}

/*
** Update the LCD display.
*/
void update_display() {
  static char freq_data[16] = { 0 };

  compute_utilization();   /* keep this current */
  
  if (update_utils) {
    /*
    ** background utilization is calculated in 1% units
    ** and displayed as a series of 'B' characters
    */
    if (last_back_util != background_utilization) {
      int bg = background_utilization;
      bar(3, bg < 19 ? bg : 19, 'B');
      last_back_util = bg;
    }
    /*
    ** foreground utilization is calculated in 1% units
    ** and displayed as a series of 'F' characters
    */
    if (last_fore_util != foreground_utilization) {
      int fg = foreground_utilization;
      if (fg == 0) {
        update_control_display2();
      } else {
        bar(2, fg < 19 ? fg : 19, 'F');
      }
      last_fore_util = fg;
    }
    update_utils = 0;
  }
  
  if (update_freq) {
    int i;
    for (i = 0; i < 16; i++) {
      freq_data[i] = profile[2*i] + profile[(2*i) + 1];
      freq_data[i] = (freq_data[i] < 15 ? freq_data[i] : 15);
      freq_data[i] = (freq_data[i] > 0 ? freq_data[i] : 0);
    }
    pixel_chart(freq_data);
    update_freq = 0;
  }
} /* update_display */
