/*********************************************************************
*                                                                    *
*                      Software License Agreement                    *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products.                                                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

#include "p30f6014.h"
#include "sound.h"

/*
 *  Each interrupt occurs every interrupt_period instr cycles.
 *    interrupt_period is calculated by init_codec
 *    samples_per_interrupt is set by init_codec
 */
int interrupt_period;
int samples_per_interrupt;
int cycles_per_interrupt;
extern int samples[];

void __attribute__((__interrupt__)) _DCIInterrupt(void) {
  static int *txBuf[] = { &TXBUF0, &TXBUF1, &TXBUF2, &TXBUF3, 0 };
  static int *rxBuf[] = { &RXBUF0, &RXBUF1, &RXBUF2, &RXBUF3, 0 };
  int i;
  static int d,sample_index;
  int T4CON_save;
  unsigned int isr_time;
       
  T4CON_save = T4CON;
  T4CONbits.TON = 0;  /* pause foreground timer */
  TMR3 = 0;
  if (status.monitor || status.play) {
    if (!curr_play_buf->status & FULL) {
      /* buffer not full, play null samples */
      TXBUF0 = 0;
      TXBUF1 = 0;
      TXBUF2 = 0;
      TXBUF3 = 0;
    } else {
      int feedback_sample;
      int sample;
      unsigned int lgain = gain + 1;

      for (i = 0; i < samples_per_interrupt; i++) {
        /* any feedback sample is crudely diminished in amplitude to simulate 
           the affect of reflection */
        feedback_sample =delay ? feedback_buf->sample[curr_play_buf->offset]: 0;
        feedback_sample >>= 1;
        sample =  curr_play_buf->sample[curr_play_buf->offset++];
        sample = (sample + feedback_sample) *lgain;
        *txBuf[i] = sample;
        /* store the first 64 samples for easy access by the power spectrum
           analyser */
        if (sample_index < 64)
          samples[sample_index++] = sample;
      }
     
      if (curr_play_buf->offset >= NUM_SAMPLES) {
        if (curr_play_buf->status & LAST) { 
          status.play = 0;
          status.stop = 1;
        }
        curr_play_buf->status = EMPTY;
        curr_play_buf->offset = 0;
        curr_play_buf = curr_play_buf->next;
        feedback_buf = curr_play_buf;
        sample_index = 0;
        d = delay;
        for (i = 0; i < d; i++) feedback_buf = feedback_buf->prev;
      }
    }
  }
  if (status.record) {
    if (curr_rec_buf->status & REC) {
      for (i = 0; i < samples_per_interrupt; i++) {
        curr_rec_buf->sample[curr_rec_buf->offset++] = *rxBuf[i];
      }

      if (curr_rec_buf->offset >= NUM_SAMPLES) {
        if (curr_rec_buf->status & LAST) {
          status.record = 0;
          status.monitor = 0;
          status.stop = 1;
        }
        curr_rec_buf->status = FULL | NEW;
        curr_rec_buf->offset = 0;
        curr_rec_buf = curr_rec_buf->next;
        curr_rec_buf->status = REC;
      }
    }
  }
  sample_count += samples_per_interrupt;
  IFS2bits.DCIIF = 0;
  cycles_per_interrupt = TMR3 + 4; /* estimate cycles in this isr */
  T4CON = T4CON_save;  /* resume foreground timer */
} 
