/*********************************************************************
*                                                                    *
*                      Software License Agreement                    *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products.                                                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

#include "p30f6014.h"
#include "lcd.h"

int buffer_count;
int max_buffer_count = 0;
char dummy;

static void lcd_wait() {
    /* the controlling pic requires 100usec to start up */
    __asm__("mov #7080, w1\n"
            "\tdo w1, 1f\n"
            "\tmov #256, w0\n"
            "\trepeat w0\n"
            "\tnop\n"
            "1:\n"
            "\tnop\n" : /* inputs */ : /* outputs */ :"w0","w1" /* clobbers */);
};

void spi_init(int poweron) {

                                     /* allow time for power-on */
  if (poweron) {
    lcd_wait();
  }
  LATGbits.LATG9 = 1;
  TRISGbits.TRISG9 = 0;
  SPI2STAT = 0;
  SPI2CON = 0x1;                    /* clk = fcy/16*/

  SPI2CONbits.SMP = 0;
  SPI2CONbits.CKE = 0;
  SPI2CONbits.CKP = 0;
  SPI2CONbits.MODE16 = 0;
  SPI2CONbits.MSTEN = 1;
  SPI2STATbits.SPIROV = 0;
  /* 
      IPC6bits.SPI2IP = 5;
  */
  IFS1bits.SPI2IF = 0;
  IEC1bits.SPI2IE = 0;                   /* disable interrupts */

  // SPI2STATbits.SPISIDL = 1;
  SPI2STATbits.SPIEN = 1;
  LATGbits.LATG9 = 0;
  while (1) {
     while (SPI2STATbits.SPITBF);
     SPI2BUF = cls;
     if (!SPI2STATbits.SPIROV) while (!SPI2STATbits.SPIRBF);
     dummy = SPI2BUF;
     if (buffer_count < 128) break;
  }
}

int write(int handle, void *buffer, unsigned int len) {
  char *c;
  int write_len = 0;
  
  SPI2STATbits.SPIROV = 0;
  LATGbits.LATG9 = 1;
  LATGbits.LATG9 = 0;
  dummy = SPI2BUF;
  c = buffer;
  for (; len; len--) {
    SPI2BUF = write_chr_next;
    while (!SPI2STATbits.SPIRBF);
    dummy = SPI2BUF;
    SPI2BUF = *c;
    while (!SPI2STATbits.SPIRBF);
    dummy = SPI2BUF;
    c++;
    write_len++;
  }
  return write_len;
}

void bar(int row, int len, char c) {
  static char clear[] = { clear_row, 0, char_pos, 0, 0 };
  
  if (row < 0) row = 0;
  if (row > 3) row = 3;
  if (len < 0) return;
  clear[1] = row;
  clear[4] = row;
  lcd_dump(clear, 5);
  for (; len; len--) {
    SPI2BUF = write_chr_next;
    while (!SPI2STATbits.SPIRBF);
    dummy = SPI2BUF;
    while (SPI2STATbits.SPITBF);
    SPI2BUF = c;
    while (!SPI2STATbits.SPIRBF);
    dummy = SPI2BUF;
  }
}

void lcd_dump(void *buffer, unsigned int len) {
  char *c;
  int first = 1;

  SPI2STATbits.SPIROV = 0;
  LATGbits.LATG9 = 1; 
  LATGbits.LATG9 = 0; 
  dummy = SPI2BUF;
  c = buffer;
  for (; len; len--) {
    SPI2BUF = *c;
    while (!SPI2STATbits.SPIRBF);
    dummy = SPI2BUF;
    if (buffer_count > max_buffer_count) max_buffer_count = buffer_count;
    first = 0;
    c++;
  }
}

void clear_lcd() {
  static char cmd[] = { cls };
  lcd_dump(cmd, 1);
}

void pixel_chart_line(char height) {
  #define offset 89
  static int line_no=0;
  static char clear_line[] = { 0xdb, 0, 0, 0xbc, 0, 0xbc, 0,  /* clear line 1*/
                               0xdb, 0, 1, 0xbc, 0, 0xbc, 0}; /* clear line 2*/
  static char draw_line[] = { 0xd7, 0, 16, 0xda, 0, 0,        /* draw line 1 */
                              0xd7, 0, 16, 0xda, 0, 0};       /* draw line 2 */


  if (height < 0) {
    line_no++;
    PORTDbits.RD3 = 0;
    return;
  }
  if (height > 15) {
    line_no++;
    PORTDbits.RD2 = 0;
    return;
  }
  clear_line[1] = offset + (line_no * 2);
  clear_line[8] = offset + (line_no * 2);
  lcd_dump(clear_line,14);
  draw_line[1] = offset + (line_no *2);
  draw_line[4] = offset + (line_no *2);
  draw_line[5] = 16 + (height);
  draw_line[7] = offset + 1 + (line_no *2);
  draw_line[10] = offset + 1 + (line_no *2);
  draw_line[11] = 16 + (height);
  lcd_dump(draw_line,12);
  line_no++;
  if (line_no > 15) line_no = 0;
}

void pixel_chart(char *arr) {
  static int how_many  =0;
  int i;

  if (how_many++ == 6) {
     lcd_wait();
  }
  for (i = 0; i < 16; i++) pixel_chart_line(*arr++);
}
  
