/*********************************************************************
*                                                                    *
*                      Software License Agreement                    *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products.                                                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

/*
** MASTERs 2004 Class 829
**
** Intro to Digital Sound Processing
**
*/

/*
** processor speed (instructions per second)
**
**   Fcy8x:  15 MIPS
**   Fcy16x: 30 MIPS
**
** comment one of the following two lines:
*/
#define Fcy Fcy8x
/* #define Fcy Fcy16x */

/* dsPICDEM crystal with 8x PLL */
#define Fcy8x (7372800 * 2)
/* dsPICDEM crystal with 16x PLL */
#define Fcy16x (7372800 * 4)

#define Iusec (const float) (Fcy * 0.000001)
#define Imsec (const float) (Fcy * 0.001)

/* timer2 period (assume 1:256 prescalar) */
#define Period_1000ms (const int) (Fcy / 256 )
#define Period_10ms (const int) ((Imsec * 10) / 256 )

/*
** Sample rates
**
** Based on the following formula, which
** is valid for the Si3000 codec. Note that
** 256 clock bits are required for each
** frame, which includes one incoming and
** one outgoing sample. BCG is the constant
** which is written to the DCICON3 register.
**
** Fs = Fcy/(256*2*(BCG + 1))
**
**    Fcy    BCG    Fs
** --------  ---  -----
** 14745600   2    9600
** 14745600   3    7200
**
** 29491200   4   11520
** 29491200   6    8228
**
*/

/*
** Bit clock constants
**
** These are values for the DCICON3 register,
** based on the sample rate table above.
*/
#define HIGH_RATE ((Fcy == Fcy8x) ? 2 : 4 )
#define LOW_RATE  ((Fcy == Fcy8x) ? 3 : 6 )

/* delay line constants */
#define NUM_SAMPLES 256
#define AUDIO_HEADER_SIZE 8
#define AUDIO_OFFSET_END (NUM_SAMPLES*2 + AUDIO_HEADER_SIZE)
#define FADE_OFFSET 16

/* delay line buffer type */
typedef struct audio_buffer {
	int status;
	int offset;
	struct audio_buffer *next;
	struct audio_buffer *prev;
	int sample[NUM_SAMPLES];
} audio_bufferT;

/* flash storage buffer type */
#define STORAGE_HEADER_SIZE 6

typedef struct storage_buffer {
	unsigned int length;
	unsigned int sample_rate;
	unsigned int bits_per_sample;
	char data[];
} storage_bufferT;

/* global status */
typedef struct status_rec {
	unsigned int record:1;      /* fill delay line from codec */
	unsigned int monitor:1;     /* monitor recording */
	unsigned int play:1;        /* play track from storage */
	unsigned int stop:1;
} statusT;

/* audio buffer status values */
#define EMPTY 0
#define REC 2
#define FULL 4
#define FIRST 8
#define LAST 16
#define NEW 32

extern volatile statusT status;
extern volatile audio_bufferT *curr_rec_buf;
extern volatile audio_bufferT *curr_play_buf;
extern volatile audio_bufferT *feedback_buf;
extern int delay;
extern int gain;
extern volatile unsigned int sample_count;
extern int utilization;
extern int interrupt_period;
extern int sample_set;
extern int samples_per_interrupt;
extern const storage_bufferT track1, track2, track3;

#include "lcd.h"
#include "main.h"
#include "frequency.h"
#include "codec.h"
#include "adc12.h"
