/*********************************************************************
*                                                                    *
*                      Software License Agreement                    *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products.                                                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

/*
** timers.c
**
** Support for dsPIC timers.
**
** Timer2 runs in 16-bit, free-running mode.
** Generates interrupts every 10 ms.
**
** Timer3 runs in 16-bit, free-running mode.
** (no interrupt)
**
** Timers 4,5 run in 32-bit, free-running mode.
** (no interrupt)
*/
#include <p30fxxxx.h>
#include "sound.h"

extern int ticks, switch_delay;

void init_timer2()
{
    T2CON = 0;
    TMR2 = 0;               /* set timer to known state */
    IFS0bits.T2IF = 0;      /* clear the interrupt flag */
    
    ticks = 0;              /* clear software counters */
    switch_delay = 0;
    
    PR2 = Period_10ms;      /* set the period */
    T2CONbits.TCKPS = 3;    /* select 1:256 prescaler */
    IPC1bits.T2IP = 3;      /* set interrupt priority level to 3 */

    IEC0bits.T2IE = 1;      /* enable timer2 interrupt */
    T2CONbits.TON = 1;      /* turn on the timer */
}

void init_timer3()
{
    T3CON = 0;
    TMR3 = 0;               /* set timer to known state */
    IFS0bits.T3IF = 0;      /* clear the interrupt flag */
    
    PR3 = 0xFFFF;           /* set the period */
    T3CONbits.TCKPS = 0;    /* no prescaler */
    IPC1bits.T3IP = 3;      /* set interrupt priority level to 3 */

    IEC0bits.T3IE = 0;      /* disable timer3 interrupt */
    T3CONbits.TON = 1;      /* turn on the timer */
}

void init_timer45()
{
    T4CON = 0;
    T5CON = 0;
    TMR4 = 0;               /* set timer to known state */
    TMR5 = 0;               /* set timer to known state */
    
    PR4 = 0xFFFF;           /* set the period */
    PR5 = 0xFFFF;           /* set the period */
    T4CONbits.TCKPS = 0;    /* no prescaler */
    T4CONbits.T32 = 1;      /* 32-bit mode */

    IPC5bits.T5IP = 0;      /* interrupt priority level = 0 (disabled) */
    IFS1bits.T5IF = 0;      /* clear the interrupt flag */
    IEC1bits.T5IE = 0;      /* disable interrupt */
    T4CONbits.TON = 0;      /* leave timer off for now*/
}

void _ISR _T2Interrupt()    /* Timer 2 ISR */
{
    IFS0bits.T2IF = 0;      /* clear interrupt flag */
    
    if (switch_delay > 0)      /* service the countdown timer */
        switch_delay--;

    if ((ticks++ % 10) == 0) {
        update_freq = 1;
        update_utils = 1;
    }

    if (ticks >= 100) {   /* blink LED 1 slowly */
        ticks = 0;
        PORTDbits.RD0 ^= 1;
    }
}
